@@ -6,13 +6,13 @@
 
 summary.manyglm <- function(object, resamp="pit.trap", test="wald", p.uni="none", nBoot=1000, cor.type=object$cor.type, show.cor = FALSE, show.est=FALSE, show.residuals=FALSE, symbolic.cor = FALSE, show.time=FALSE, ... ) 
 {
-    tol = object$tol
     allargs <- match.call(expand.dots = FALSE)
     dots <- allargs$...
-    if ("ld.perm" %in% names(dots)) ld.perm <- dots$ld.perm
-    else ld.perm <- FALSE
-    if ("filename" %in% names(dots)) filename <- dots$filename
-    else filename <- NULL
+    tol = 1e-4
+    if ("rep.seed" %in% names(dots)) rep.seed <- dots$rep.seed
+    else rep.seed <- FALSE
+    if ("bootID" %in% names(dots)) bootID <- dots$bootID
+    else bootID <- NULL
 
     if (show.time==FALSE) st=0
     else st=1
@@ -23,20 +23,15 @@ summary.manyglm <- function(object, resamp="pit.trap", test="wald", p.uni="none"
     }
     if (any(class(object)=="manylm")) {
         if ( test == "LR" ) 
-           return(summary.manylm(object, resamp=resamp, test="LR", p.uni=p.uni, nBoot=nBoot, cor.type=cor.type, show.cor=show.cor, show.est=show.est, show.residuals=show.residuals, symbolic.cor=symbolic.cor, tol=tol, ld.perm=ld.perm, filename=filename, ... ))
+           return(summary.manylm(object, resamp=resamp, test="LR", p.uni=p.uni, nBoot=nBoot, cor.type=cor.type, show.cor=show.cor, show.est=show.est, show.residuals=show.residuals, symbolic.cor=symbolic.cor, tol=tol, bootID=bootID, ... ))
 	else {   
 	   warning("For an manylm object, only the likelihood ratio test and F test are supported. So the test option is changed to `'F''. ")
-           return(summary.manylm(object, resamp=resamp, test="F", p.uni=p.uni, nBoot=nBoot, cor.type=cor.type, show.cor=show.cor, show.est=show.est, show.residuals=show.residuals, symbolic.cor=symbolic.cor, tol=tol, ld.perm=ld.perm, filename=filename, ... ))
+           return(summary.manylm(object, resamp=resamp, test="F", p.uni=p.uni, nBoot=nBoot, cor.type=cor.type, show.cor=show.cor, show.est=show.est, show.residuals=show.residuals, symbolic.cor=symbolic.cor, tol=tol, bootID=bootID, ... ))
 	}   
     }
     else if (!any(class(object)=="manyglm"))
        stop("The function 'summary.manyglm' can only be used for a manyglm or manylm object.")
     
-    # ld.perm and filename for debug use only
-    # ld.perm=TRUE load bootID from file
-#    ld.perm = TRUE
-#    filename = NULL 
-
     nRows = nrow(object$y)
     nVars = ncol(object$y)
     nParam = ncol(object$x)
@@ -55,8 +50,8 @@ summary.manyglm <- function(object, resamp="pit.trap", test="wald", p.uni="none"
     else if (object$family == "binomial") familynum <- 3 
     else stop("'family' not defined. Choose one of 'poisson', 'negative.binomial', 'binomial' for an manyglm object") 
 
-    if (object$phi.method == "ML") methodnum <- 0
-    else if (object$phi.method == "Chi2") methodnum <- 1 
+    if (object$theta.method == "ML") methodnum <- 0
+    else if (object$theta.method == "Chi2") methodnum <- 1 
     else stop("'method' not defined. Choose one of 'ML', 'Chi2' for an manyglm object") 
     if (substr(resamp,1,1)=="c") resampnum <- 0  #case
     # To exclude case resampling
@@ -71,9 +66,9 @@ summary.manyglm <- function(object, resamp="pit.trap", test="wald", p.uni="none"
 
     # allows case and parametric bootstrap only for binomial regression
     if (familynum == 3 && ( (resampnum !=5) && (resampnum!=8) ) ) {
-       warning("'montecarlo' or 'pit.trap' should be used for binomial regression. Resampling option is changed to 'pit.trap'.")
-       resamp <- "pit.trap"
-       resampnum <- 8 
+       warning("'montecarlo' or 'pit.trap' should be used for binomial regression..")
+      # resamp <- "pit.trap"
+      # resampnum <- 8 
     }
   
     if (substr(test,1,1) == "w") testnum <- 2 # wald
@@ -86,15 +81,6 @@ summary.manyglm <- function(object, resamp="pit.trap", test="wald", p.uni="none"
     else if (cor.type == "shrink") corrnum <- 2
     else stop("'cor.type' not defined. Choose one of 'I', 'R', 'shrink'")
 
-    if (ld.perm && !is.null(filename)) {
-        bootID <- as.matrix(read.table(filename), nrow=nBoot, ncol=nRows)
-        rep <- 1
-    }
-    else {
-        bootID <- c(FALSE)
-        rep <- 0
-    }
-
     if (substr(p.uni,1,1) == "n"){
        pu <- 0
        calc.pj <- FALSE
@@ -110,7 +96,18 @@ summary.manyglm <- function(object, resamp="pit.trap", test="wald", p.uni="none"
     } else
        stop("'p.uni' not defined. Choose one of 'single', 'adjusted', 'unadjusted', 'none'.")
 
-    if (corrnum == 2 | resampnum==5 ) {
+ if (!is.null(bootID)) {
+     nBoot<-dim(bootID)[2]
+     if (is.integer(bootID)) {
+        cat(paste("Input bootID matrix being used for testing.","\n"))       
+     }
+     else {
+        bootID <- NULL
+	cat(paste("Invalid bootID. Calculate bootID matrix on the fly.","\n"))
+     }
+ } 
+
+  if (corrnum == 2 | resampnum==5 ) {
        # get the shrinkage estimates
        shrink.param <- c(rep(NA,nParam+2))
        tX <- matrix(1, nRows, 1)
@@ -128,9 +125,9 @@ summary.manyglm <- function(object, resamp="pit.trap", test="wald", p.uni="none"
     else if (corrnum == 0) shrink.param <- c(rep(1,nParam+2))
     else if (corrnum == 1) shrink.param <- c(rep(0,nParam+2))
     
-    modelParam <- list(tol=tol, regression=familynum, estimation=methodnum, stablizer=0, n=object$K)
+    modelParam <- list(tol=tol, regression=familynum, estimation=methodnum, stablizer=0, n=object$K, maxiter=object$maxiter)
     # note that nboot excludes the original dataset
-    testParams <- list(tol=tol, nboot=nBoot-1, cor_type=corrnum, test_type=testnum, resamp=resampnum, reprand=rep, punit=pu, showtime=st)
+    testParams <- list(tol=tol, nboot=nBoot-1, cor_type=corrnum, test_type=testnum, resamp=resampnum, reprand=rep.seed, punit=pu, showtime=st)
 
     ######## Call Summary Rcpp #########
     val <- .Call("RtoGlmSmry", modelParam, testParams, Y, X, bootID, shrink.param, PACKAGE="mvabund")
