@@ -21,6 +21,7 @@ RcppExport SEXP RtoGlm(SEXP params, SEXP Ysexp, SEXP Xsexp)
     mm.estiMethod = as<unsigned int>(rparam["estimation"]);
     mm.varStab = as<unsigned int>(rparam["stablizer"]);
     mm.n = as<unsigned int>(rparam["n"]);
+    mm.maxiter = as<unsigned int>(rparam["maxiter"]);
 // for debug
 //    Rprintf("tol=%g, model=%d, estiMethod=%d, varStab=%d\n", mm.tol, mm.model, mm.estiMethod, mm.varStab);
 
@@ -79,7 +80,7 @@ RcppExport SEXP RtoGlm(SEXP params, SEXP Ysexp, SEXP Xsexp)
 //    Rprintf("Time elapsed: %d hr %d min %d sec (%d seconds)\n", hours, min, sec, dif);
 
     // Wrap the glm object with Rcpp 
-    NumericVector phi(glmPtr[mtype]->phi, glmPtr[mtype]->phi+nVars);
+    NumericVector theta(glmPtr[mtype]->theta, glmPtr[mtype]->theta+nVars);
     NumericVector ll(glmPtr[mtype]->ll, glmPtr[mtype]->ll+nVars);
     NumericVector aic(glmPtr[mtype]->aic, glmPtr[mtype]->aic+nVars);
     NumericVector dev(glmPtr[mtype]->dev, glmPtr[mtype]->dev+nVars);
@@ -131,7 +132,7 @@ RcppExport SEXP RtoGlm(SEXP params, SEXP Ysexp, SEXP Xsexp)
 	 _["sqrt.1_Hii"] = sqrt1_Hii,
          _["var.estimator"] = Vars,
 	 _["sqrt.weight"] = wHalf,
-	 _["phi"] = phi,
+	 _["theta"] = theta,
 	 _["two.loglike"] = ll,
 	 _["deviance"] = dev,
 	 _["aic"] = aic,
