@@ -22,6 +22,7 @@ RcppExport SEXP RtoGlmSmry(SEXP mpar, SEXP tpar, SEXP Ysexp, SEXP Xsexp,
     mm.estiMethod = as<unsigned int>(sparam["estimation"]);
     mm.varStab = as<unsigned int>(sparam["stablizer"]);
     mm.n = as<unsigned int>(sparam["n"]);
+    mm.maxiter = as<unsigned int>(sparam["maxiter"]);
 
     List rparam(tpar);
     // pass parameters
@@ -116,7 +117,8 @@ RcppExport SEXP RtoGlmSmry(SEXP mpar, SEXP tpar, SEXP Ysexp, SEXP Xsexp,
     unsigned int hours = floor((double)dif/(double)3600);
     unsigned int min = floor((double)(dif-hours*3600)/(double)60);
     unsigned int sec = dif - hours*3600 - min*60;
-    Rprintf("Time elapsed: %d hr %d min %d sec\n", hours, min, sec);
+    if (tm.showtime==TRUE)
+       Rprintf("Time elapsed: %d hr %d min %d sec\n", hours, min, sec);
 
     // Wrap gsl vectors with Rcpp 
     double multstat, Pmultstat;
