@@ -13,13 +13,17 @@ glm::glm(const reg_Method *mm)
    : mmRef(mm), Yref(NULL), Xref(NULL), Oref(NULL), 
      Beta(NULL), varBeta(NULL), Mu(NULL), Eta(NULL), Res(NULL), 
      Var(NULL), wHalf(NULL), sqrt1_Hii(NULL), PitRes(NULL), 
-     phi(NULL), ll(NULL), dev(NULL), aic(NULL), iterconv(NULL) 
+     theta(NULL), ll(NULL), dev(NULL), aic(NULL), iterconv(NULL) 
 { 
+     maxth = 100;
      n=mmRef->n;
-     maxiter = 50;
-     eps = 1e-5;
-     if (mmRef->model==BIN) maxtol=n-eps;
-     else maxtol = 1/eps;
+     maxiter = mmRef->maxiter;
+     // Error terms
+     eps = mmRef->tol;   
+     // Valid data range of mu or y
+     mintol = mmRef->tol; 
+     if (mmRef->model==BIN) maxtol=n-mintol; 
+     else maxtol = 1/mintol;
 }
 
 PoissonGlm::PoissonGlm(const reg_Method *mm):glm(mm){
@@ -73,8 +77,8 @@ void glm::releaseGlm(void)
         gsl_matrix_free(sqrt1_Hii);
     if (PitRes!=NULL)
         gsl_matrix_free(PitRes);
-    if (phi!=NULL)
-        delete[] phi;
+    if (theta!=NULL)
+        delete[] theta;
     if (ll!=NULL)	
         delete[] ll;
     if (dev!=NULL)	
@@ -94,7 +98,7 @@ void glm::initialGlm(gsl_matrix *Y, gsl_matrix *X, gsl_matrix *O, gsl_matrix *B)
     nParams = X->size2;
 
     unsigned int i, j;
-    phi = new double [nVars];
+    theta = new double [nVars];
     ll = new double [nVars];
     dev = new double [nVars];
     aic = new double [nVars];
@@ -122,7 +126,7 @@ void glm::initialGlm(gsl_matrix *Y, gsl_matrix *X, gsl_matrix *O, gsl_matrix *B)
     gsl_matrix_set_zero (varBeta);
 
     for (j=0; j<nVars; j++) {
-        phi[j] = 0;
+        theta[j] = maxtol;  // i.e. phi=0
         ll[j] = 0;
         dev[j] = 0;
         aic[j] = 0;
@@ -138,35 +142,61 @@ void glm::initialGlm(gsl_matrix *Y, gsl_matrix *X, gsl_matrix *O, gsl_matrix *B)
 //
 //  Use binomial$initialize: MuStart = (Y+0.5)/2 
 //  It seems to work for poisson and negative.binomial as well
+    double LinAdjust, ScaleAdjust;
+    double eij;
+    if (mmRef->model==BIN) {
+        LinAdjust=0.5;
+	ScaleAdjust=0.5;
+    }
+    else {
+        LinAdjust=0.1;
+	ScaleAdjust=1;
+    }
     if (B!=NULL) {
        gsl_matrix_memcpy(Beta, B);
        gsl_blas_dgemm (CblasNoTrans, CblasNoTrans,1.0,X,Beta,0.0,Eta);
        for (i=0; i<nRows; i++)
-       for (j=0; j<nVars; j++)
-           gsl_matrix_set(Mu, i, j, invLink(gsl_matrix_get(Eta, i, j)));
-    }  
+       for (j=0; j<nVars; j++) {
+           eij = gsl_matrix_get(Eta, i, j);
+	   // to avoid nan
+           if (eij>link(maxtol)) eij = link(maxtol);
+	   if (eij<link(mintol)) eij = link(mintol);
+	   gsl_matrix_set(Eta, i, j, eij);
+           gsl_matrix_set(Mu, i, j, invLink(eij));
+       }  
+    }
     else if (O!=NULL) {
+       gsl_matrix_set_zero(Beta); 
        gsl_matrix_memcpy(Eta, O);
        for (i=0; i<nRows; i++)
-       for (j=0; j<nVars; j++)
-           gsl_matrix_set(Mu, i, j, invLink(gsl_matrix_get(Eta, i, j)));
-       gsl_matrix_set_zero(Beta);   
-    }
-    else {
-       gsl_matrix_memcpy(Mu, Y);
-       gsl_matrix_add_constant(Mu, 0.5);
-       gsl_matrix_scale(Mu, 0.5);
+       for (j=0; j<nVars; j++) {
+           eij = gsl_matrix_get(Eta, i, j);
+	   // to avoid nan
+           if (eij>link(maxtol)) eij = link(maxtol);
+	   if (eij<link(mintol)) eij = link(mintol);
+	   gsl_matrix_set(Eta, i, j, eij);
+           gsl_matrix_set(Mu, i, j, invLink(eij));
+       }   
+    } 
+    else { 
+       gsl_matrix_memcpy(Mu, Yref);
+       gsl_matrix_add_constant(Mu, LinAdjust);
+       gsl_matrix_scale(Mu, ScaleAdjust);
+//       gsl_matrix_set_zero (Eta); // intercept
        for (i=0; i<nRows; i++)
-       for (j=0; j<nVars; j++)
-          gsl_matrix_set(Eta, i, j, link(gsl_matrix_get(Mu, i, j)));
+       for (j=0; j<nVars; j++) {
+          eij = link(gsl_matrix_get(Mu, i, j));
+          if (eij>link(maxtol)) eij = link(maxtol);
+	  if (eij<link(mintol)) eij = link(mintol);
+          gsl_matrix_set(Eta, i, j, eij);
+	  gsl_matrix_set(Mu, i, j, invLink(eij));
+       }	  
        gsl_matrix_set_zero (Beta); // intercept
        gsl_vector_view b0=gsl_matrix_column(Beta, 0);
        gsl_vector_set_all(&b0.vector, 1.0);
+   //    printf("LinAdjust=%.2f, ScaleAdjust=%.2f\n", LinAdjust, ScaleAdjust);
     }
 
-//   displaymatrix(Eta, "Eta");
-//   displaymatrix(Mu, "Mu");
-
    rdf = nRows - nParams;
 }
 
@@ -187,7 +217,7 @@ int glm::copyGlm(glm *src)
     gsl_matrix_memcpy(PitRes, src->PitRes);
     
     for (unsigned int i=0; i<nVars; i++) {
-        phi[i] = src->phi[i];	
+        theta[i] = src->theta[i];	
         ll[i] = src->ll[i];
         dev[i] = src->dev[i];
         iterconv[i] = src->iterconv[i];
@@ -213,30 +243,30 @@ int PoissonGlm::EstIRLS(gsl_matrix *Y, gsl_matrix *X, gsl_matrix *O, gsl_matrix
     gsl_matrix *XwX = gsl_matrix_alloc(nParams, nParams);   
 
     for (j=0; j<nVars; j++) {
-       if ( a!=NULL ) phi[j]=a[j]; 
+       if ( a!=NULL ) theta[j]=a[j]; 
        // estimate mu and beta   
-       iterconv[j] = betaEst(j, maxiter, &tol, phi[j]); 
+       iterconv[j] = betaEst(j, maxiter, &tol, theta[j]); 
        if (iterconv[j]>maxiter) 
            printf("EstIRLS reached max iterations\n");
        gsl_matrix_memcpy (WX, X);
        for (i=0; i<nRows; i++) {
             mij = gsl_matrix_get(Mu, i, j);
             // get variance
-            vij = varfunc( mij, phi[j] );
+            vij = varfunc( mij, theta[j] );
             gsl_matrix_set(Var, i, j, vij); 
             // get weight
-            wij = sqrt(weifunc(mij, phi[j]));  
+            wij = sqrt(weifunc(mij, theta[j]));  
             gsl_matrix_set(wHalf, i, j, wij);             
             // get (Pearson) residuals
             yij = gsl_matrix_get(Y, i, j);
             gsl_matrix_set(Res, i, j, (yij-mij)/sqrt(vij));        
             // get PIT residuals for discrete data
             wei = gsl_rng_uniform_pos (rnd); // wei ~ U(0, 1)
-            uij = wei*cdf(yij, mij, phi[j]);
-            if (yij>0) uij=uij+(1-wei)*cdf((yij-1),mij,phi[j]);
+            uij = wei*cdf(yij, mij, theta[j]);
+            if (yij>0) uij=uij+(1-wei)*cdf((yij-1),mij,theta[j]);
             gsl_matrix_set(PitRes, i, j, uij);
             // get elementry log-likelihood	   
-            ll[j] = ll[j] + llfunc( yij, mij, phi[j] );
+            ll[j] = ll[j] + llfunc( yij, mij, theta[j]);
             // W^1/2 X
             Xwi = gsl_matrix_row (WX, i);
             gsl_vector_scale(&Xwi.vector, wij);            
@@ -285,15 +315,14 @@ int PoissonGlm::EstIRLS(gsl_matrix *Y, gsl_matrix *X, gsl_matrix *O, gsl_matrix
 }
 
 
-int PoissonGlm::betaEst( unsigned int id, unsigned int iter, double *tol, double phi )
+int PoissonGlm::betaEst( unsigned int id, unsigned int iter, double *tol, double th)
 {
    gsl_set_error_handler_off();
    int status, isValid;
-   unsigned int i, j, ngoodobs;
-   unsigned int step, step1; 
-   double wij, zij, eij, mij, yij;   
+   unsigned int j, ngoodobs;
+   unsigned int i, step, step1; 
+   double wij, zij, eij, mij, yij, bij;   
    double dev_old, dev_grad=1.0;
-   unsigned int *good = new unsigned int [nRows];
    gsl_vector_view Xwi;
    gsl_matrix *WX, *XwX;
    gsl_vector *z, *Xwz;
@@ -301,65 +330,32 @@ int PoissonGlm::betaEst( unsigned int id, unsigned int iter, double *tol, double
    gsl_vector_view bj=gsl_matrix_column (Beta, id);
 
    // Main Loop of IRLS begins
+   z = gsl_vector_alloc(nRows);
+   WX = gsl_matrix_alloc(nRows, nParams); 
    XwX = gsl_matrix_alloc(nParams, nParams);
    Xwz = gsl_vector_alloc(nParams);
    step=0;
    *tol = 1.0;
    gsl_vector_memcpy (coef_old, &bj.vector);
    while ( step<iter ) {
-       // Decide good observations
-       ngoodobs=0;
-       for (i=0; i<nRows; i++) {
-           good[i]=TRUE;
-           eij = gsl_matrix_get(Eta, i, id);
-           mij = gsl_matrix_get(Mu, i, id);  
-           if ((mij<eps)|(mij>maxtol)) 
-	      good[i]=FALSE;
-  //         if ((mmRef->model==BIN)&(rcpLinkDash(mij)<eps*maxtol))
-  //            good[i]=FALSE;
-  //         if ((mmRef->model!=BIN)&(rcpLinkDash(mij)<eps))
-           if (rcpLinkDash(mij)<eps) good[i]=FALSE;
-           if (good[i]==TRUE) ngoodobs++;
-       }  
-    
-       if (ngoodobs==0) {
- //          printf("mij(rcpLinkDash)\n");
-//           for (i=0; i<nRows; i++) {
-//              mij = gsl_matrix_get(Mu, i, id);
- //             printf("%.8f(%.8f)\t", mij, rcpLinkDash(mij));
- //          }
- //          printf("\nError: ngoodobs=0\n");      
-          for (i=0; i<nRows; i++) {
-              good[i]=TRUE;
-	      if (rcpLinkDash(mij)<eps*0.1) good[i]=FALSE;
-	      if (good[i]==TRUE) ngoodobs++;
-	  }
-       }
-
-//       gsl_vector_set_zero (&wj.vector);
-       z = gsl_vector_alloc(ngoodobs);
-       WX = gsl_matrix_alloc(ngoodobs, nParams); 
-       j=0;  // index for z and WX
        for (i=0; i<nRows; i++) { // (y-m)/g'
-           if ( good[i]==TRUE ) {
-              // z=xb+(y-m)/g'-o
-              yij = gsl_matrix_get(Yref, i, id);
-              mij = gsl_matrix_get(Mu, i, id);
-              eij = gsl_matrix_get(Eta, i, id);
-              zij = eij + (yij-mij)/rcpLinkDash(mij);
-              if (Oref!=NULL) 
-                 zij = zij - gsl_matrix_get(Oref, i, id);
-              // wt=sqrt(weifunc);
-              wij = sqrt(weifunc(mij, phi));
-              // W^1/2*z[good]
-              gsl_vector_set(z, j, wij*zij); 
-              // W^1/2*X[good]
-              Xwi = gsl_matrix_row (Xref, i);
-              gsl_matrix_set_row (WX, j, &Xwi.vector);
-              Xwi = gsl_matrix_row (WX, j);
-              gsl_vector_scale(&Xwi.vector, wij); 
-              j++;
-          }
+           yij = gsl_matrix_get(Yref, i, id);
+           eij = gsl_matrix_get(Eta, i, id);
+           mij = gsl_matrix_get(Mu, i, id);
+        //   if (mij<mintol) mij=mintol;
+        //   if (mij>maxtol) mij=maxtol;
+           zij = eij + (yij-mij)/rcpLinkDash(mij);
+           if (Oref!=NULL) 
+              zij = zij - gsl_matrix_get(Oref, i, id);
+           // wt=sqrt(weifunc);
+           wij = sqrt(weifunc(mij, th));
+           // W^1/2*z[good]
+           gsl_vector_set(z, i, wij*zij); 
+           // W^1/2*X[good]
+           Xwi = gsl_matrix_row (Xref, i);
+           gsl_matrix_set_row (WX, i, &Xwi.vector);
+           Xwi = gsl_matrix_row (WX, i);
+           gsl_vector_scale(&Xwi.vector, wij); 
        }
        // in glm2, solve WXb=Wz, David suggested not good 
        // So back to solving X'WXb=X'Wz
@@ -367,18 +363,35 @@ int PoissonGlm::betaEst( unsigned int id, unsigned int iter, double *tol, double
        gsl_blas_dsyrk (CblasLower,CblasTrans,1.0,WX,0.0,XwX); 
        if (calcDet(XwX)<eps){ // test if singular or nan
           gsl_vector_view dj=gsl_matrix_diagonal(XwX);
-          gsl_vector_add_constant(&dj.vector, eps);
+	  gsl_vector_add_constant(&dj.vector, eps);
        }   
        status=gsl_linalg_cholesky_decomp(XwX);
        if (status) {
           printf("Singular XwX in betaEst\n");
+	  printf("theta=%.4f, calcDet(XwX)=%.4f\n", th, calcDet(XwX));
+	  displaymatrix(XwX, "XwX"); 
        }
        gsl_blas_dgemv(CblasTrans,1.0,WX,z,0.0,Xwz);
        gsl_linalg_cholesky_solve (XwX, Xwz, &bj.vector);
 
+   // Debug for nan
+/*   if (gsl_vector_get(&bj.vector, 1)!=gsl_vector_get(&bj.vector, 1)) {
+       displayvector(&bj.vector, "bj");
+       displayvector(z, "z");
+       gsl_vector_view mj=gsl_matrix_column(Mu, id);
+       displayvector(&mj.vector, "mj");
+       printf("weight\n");
+       for (i=0; i<nRows; i++){
+           printf("%.4f ", sqrt(weifunc(mij, th)));
+       }
+       printf("\n");
+       displaymatrix(XwX, "XwX");
+       exit(-1);
+   }   
+*/
        // Given bj, update eta, mu
        dev_old = dev[id];
-       isValid=predict(bj, coef_old, id, phi);
+       isValid=predict(bj, id, th);
        dev_grad=(dev[id]-dev_old)/(ABS(dev[id])+0.1);
        *(tol)=ABS(dev_grad);  
 
@@ -386,41 +399,28 @@ int PoissonGlm::betaEst( unsigned int id, unsigned int iter, double *tol, double
        // If divergent or increasing deviance, half step
        // (step>1) -> (step>0) gives weired results for NBin fit       
        // below works for boundary values, esp BIN fit but not NBin fit
-       while (((dev[id]>10)|(dev_grad>-0.25))&(step>1)){
+       while ((dev_grad>eps)&(step>1)){
             gsl_vector_add (&bj.vector, coef_old);
             gsl_vector_scale (&bj.vector, 0.5);
             dev_old=dev[id];
-            isValid=predict(bj, coef_old, id, phi);
+            isValid=predict(bj, id, th);
             dev_grad=(dev[id]-dev_old)/(ABS(dev[id])+0.1); 
             *tol=ABS(dev_grad);
             if (*tol<eps) break;
             step1++;
-            if (step1>maxiter) {
-               printf("\t Internal loop reached max iter, d_dev=%.4f\n", dev_grad);
+            if (step1>10) {
+            //   printf("\t Half step stopped at iter %d: gradient=%.8f\n", step1, dev_grad);
                break;
             }
        }
-
-       gsl_vector_free(z);
-       gsl_matrix_free(WX); 
-       if ((isValid==TRUE))
-          gsl_vector_memcpy (coef_old, &bj.vector);
+       if (isValid==TRUE) gsl_vector_memcpy (coef_old, &bj.vector);
+      
        step++;
        if (*tol<eps) break;
-
    } 
 
-//   if ((step>1)&(isValid==FALSE)) { // restore previous valid one
-//      update(coef_old, id);
-//      dev[id]=0;
-//      for ( i=0; i<nRows; i++ ) {
-//          yij = gsl_matrix_get(Yref, i, id);
-//          mij = gsl_matrix_get(Mu, i, id);         
-//          dev[id] = dev[id] + devfunc(yij, mij, phi);
-//      }
-//   }
-
-   delete[] good;
+   gsl_vector_free(z);
+   gsl_matrix_free(WX); 
    gsl_matrix_free(XwX); 
    gsl_vector_free(Xwz); 
    gsl_vector_free(coef_old); 
@@ -440,12 +440,15 @@ int PoissonGlm::update(gsl_vector *bj, unsigned int id)
        gsl_blas_ddot (&xi.vector, bj, &eij);
        if (Oref!=NULL) 
           eij = eij+gsl_matrix_get(Oref, i, id);
-       mij = invLink(eij);
-       if ((mij<eps)|(mij>maxtol)){
-          mij=(mij<eps)?eps:((mij>maxtol)?maxtol:mij);
-          eij=link(mij);
-          isValid = FALSE;
+       if (eij>link(maxtol)) { // to avoid nan;
+          eij = link(maxtol);
+	  isValid=FALSE;
        }
+       if (eij<link(mintol)){
+          eij = link(mintol);
+	  isValid=FALSE;
+       }
+       mij = invLink(eij);
        gsl_matrix_set(Eta, i, id, eij);
        gsl_matrix_set(Mu, i, id, mij);
    } 
@@ -453,44 +456,19 @@ int PoissonGlm::update(gsl_vector *bj, unsigned int id)
    return isValid;
 }
 
-int PoissonGlm::predict(gsl_vector_view bj, gsl_vector *coef_old, unsigned int id, double phi) 
+int PoissonGlm::predict(gsl_vector_view bj, unsigned int id, double th) 
 {
     unsigned int i;
     double yij, mij;
     
     int isValid=update(&bj.vector, id);
 
-/*  unsigned int step=0;
-    double bij, diff, delta;
-    while ( isValid==FALSE ) {
-         // Step too large, half step 
-         gsl_vector_add (&bj.vector, coef_old);
-         gsl_vector_scale (&bj.vector, 0.5);
-         isValid = update(&bj.vector, id);
-         step++;
-         // If difference small, stop
-         delta=0;
-         for (i=0; i<nParams; i++) {
-             bij = gsl_vector_get(&bj.vector, i);
-             diff=bij-gsl_vector_get(coef_old, i);
-             delta=delta+ABS(diff/(ABS(bij)+0.1));
-         }
-         if (delta<eps) { // accuracy 
-//            printf("\t\t Internal loop II(%d), delta=%.4f\n", step, delta);
-             break;
-         }
-         if (step>maxiter) {
-             printf("\t\t Internal loop II(%d) reached maximum iter %d, delta=%.4f\n", isValid, step, delta);
-             break;
-         }
-     }
-*/
      // Given valid mj, estimate deviance
      dev[id]=0;
      for ( i=0; i<nRows; i++ ) {
          yij = gsl_matrix_get(Yref, i, id);
          mij = gsl_matrix_get(Mu, i, id);         
-         dev[id] = dev[id] + devfunc(yij, mij, phi);
+         dev[id] = dev[id] + devfunc(yij, mij, th);
      }
 
      return isValid;
@@ -506,7 +484,7 @@ int NBinGlm::nbinfit(gsl_matrix *Y, gsl_matrix *X, gsl_matrix *O, gsl_matrix *B)
     gsl_rng *rnd=gsl_rng_alloc(gsl_rng_mt19937);
     unsigned int i, j, isConv;
     double yij, mij, vij, hii, uij, wij, wei;
-    double a, phi_old, fA, fAdash, tol;
+    double th, th0, tol, d1, lm0, lm;
 //    double tol_grad, tol_old;
     int status;
     gsl_vector_view b0j, m0j, e0j, v0j;
@@ -532,73 +510,54 @@ int NBinGlm::nbinfit(gsl_matrix *Y, gsl_matrix *X, gsl_matrix *O, gsl_matrix *B)
 	gsl_matrix_set_col(varBeta, j, &v0j.vector);
         dev[j] = fit0.dev[j];
 
-        // Get initial phi estimates 
-        iterconv[j]=0;
+        // Get initial theta estimates
+	tol = 1;
+        iterconv[j]=0;	
         if (mmRef->estiMethod==CHI2) {
-            a = fit0.getDisper(j); 
-            phi[j] = (a<1)? 0:1/a;
-	    isConv = (a<1)? TRUE:FALSE;
-	    while (isConv != TRUE) {
-                iterconv[j]++;
-	        betaEst(j, 1, &tol, phi[j]);  // 1-step beta
-		phi[j] = phi[j]*getDisper(j); // 1-step phi
-	        if (tol<eps) break; // Normal break;
-                if ( (phi[j]<0) | (phi[j]!=phi[j])) break; // invalide phi 
-                if (iterconv[j]>maxiter) {
-                   printf("NBinFit(Chi2) reached maximum iter\n"); 
-                   break;
-                }
-            }
-        }   
-        else if (mmRef->estiMethod==NEWTON) {         
-            getfAfAdash(eps, j, &fA, &fAdash);
-            a = (fA>0)?-fA/fAdash:0;
-            phi[j] = (a<0)? eps:a; 
-	    isConv = (fA>0)? FALSE:TRUE;
-            phi_old = phi[j];
-	    while ( isConv != TRUE ) {
-                iterconv[j]++;	    
-	        betaEst(j, 1, &tol, phi[j]); // 1-step beta (better than m-step beta)
-                getfAfAdash(phi[j], j, &fA, &fAdash); // 1-step phi
-                if ( (ABS(fAdash)<eps)|(ABS(fA/fAdash)>0.5) ) 
-                   phi[j] = (phi[j]+phi_old)/2; // half step
-                else {
-                   phi_old = phi[j];
-                   phi[j] = phi[j]-fA/fAdash;
-                }        
-	        if ( tol<eps ) break; // Normal break;
-                if ( (phi[j]<0) | (phi[j]!=phi[j]) ) break; // invalid phi
-                if (iterconv[j]>maxiter) {                
-//                   printf("NBinFit(ML) reached %d, phi[j]=%.4f, fA=%.4f, (.)=%.4f, tol=%.6f, tol_old=%.6f, tol_grad=%.6f\n", iterconv[j], phi[j], fA, (fA/fAdash), tol, tol_old, tol_grad); 
-                   break; 
-                } 
+           th = getDisper(j, 0); 
+	   while ( (tol>eps)&(iterconv[j]<maxiter) ) {
+               iterconv[j]++;
+	       betaEst(j, 1, &tol, th);  // 1-step beta
+	       th = th*getDisper(j, th); // 1-step theta 
            }
-       }  
+        }   
+        else if (mmRef->estiMethod==NEWTON) {
+	   while ( (tol>eps)&(iterconv[j]<maxiter) ) {
+               iterconv[j]++;
+          //     th0 = th;
+	       th = thetaML(0, j, 10);
+               betaEst(j, 10, &tol, th);	
+           //    lm0 = lm;
+	   //    lm = 0;
+	   //    for (i=0; i<nRows; i++) {
+	   //        yij = gsl_matrix_get(Y, i, j);
+	   //        mij = gsl_matrix_get(Mu, i, j);
+           //        lm = lm+llfunc(yij, mij, th);
+	   //     }		
+           //    tol = ABS(lm-lm0)/sqrt(2*MAX(1,rdf))+ABS(th0-th);
+           //    tol = tol + ABS(th0-th);	       
+           }
+       }       
+     //  if (iterconv[j]==maxiter) {
+     //      printf("iter[%d]=%d, theta[%d]=%.4f, tol=%.4f, dev[%d]=%.4f\n", j, iterconv[j], j, th, tol, j, dev[j]);
 
-       // restore poisson if phi[j]<0 or nan
-       if ( (phi[j] < 0) | (phi[j]!=phi[j]) ) { 
-            phi[j]=0;
-            gsl_matrix_set_col (Beta, j, &b0j.vector);
-            gsl_matrix_set_col (Mu, j, &m0j.vector);
-            gsl_matrix_set_col (Eta, j, &e0j.vector);
-            gsl_matrix_set_col (varBeta, j, &v0j.vector);
-            dev[j]=fit0.dev[j];
-       }
+     //  }	   
        // other properties based on mu and phi
-       gsl_matrix_memcpy(WX, Xref);
+       theta[j] = th;
+       gsl_matrix_memcpy(WX, Xref);        
        for (i=0; i<nRows; i++) {
            yij = gsl_matrix_get(Y, i, j);
            mij = gsl_matrix_get(Mu, i, j);
-           vij = varfunc( mij, phi[j] );
+           vij = varfunc( mij, th);
            gsl_matrix_set(Var, i, j, vij); 
-           wij = sqrt(weifunc(mij, phi[j]));
+           wij = sqrt(weifunc(mij, th));
            gsl_matrix_set(wHalf, i, j, wij); 
            gsl_matrix_set(Res, i, j, (yij-mij)/sqrt(vij));        
-           ll[j] = ll[j] + llfunc( yij, mij, phi[j] );
+           ll[j] = ll[j] + llfunc( yij, mij, th);
            // get PIT residuals for discrete data
            wei = gsl_rng_uniform_pos (rnd); // wei ~ U(0, 1)
-           uij = wei*cdf(yij, mij, phi[j]);
-           if (yij>0) uij=uij+(1-wei)*cdf((yij-1),mij,phi[j]);           
+           uij=wei*cdf(yij, mij, th);
+           if (yij>0) uij=uij+(1-wei)*cdf((yij-1),mij,th); 
            gsl_matrix_set(PitRes, i, j, uij);
            // W^1/2 X
            Xwi = gsl_matrix_row (WX, i);
@@ -609,7 +568,7 @@ int NBinGlm::nbinfit(gsl_matrix *Y, gsl_matrix *X, gsl_matrix *O, gsl_matrix *B)
        // X^T * W * X
        gsl_matrix_set_zero (XwX);
        gsl_blas_dsyrk (CblasLower, CblasTrans, 1.0, WX, 0.0, XwX);
-       if (calcDet(XwX)<eps) { //XwX + eps*I
+       if (calcDet(XwX)<eps) {
           dj=gsl_matrix_diagonal(XwX);
           gsl_vector_add_constant(&dj.vector, eps);
        }       
@@ -617,6 +576,7 @@ int NBinGlm::nbinfit(gsl_matrix *Y, gsl_matrix *X, gsl_matrix *O, gsl_matrix *B)
        if (status) {
           displaymatrix(XwX, "XwX");
           printf("Singular info mat in NBinFit - calcDet(XwX)=%.4f\n", calcDet(XwX)); 
+//          exit(-1);
        }
        gsl_linalg_cholesky_invert (XwX); // (X'WX)^-1
 
@@ -649,7 +609,7 @@ int NBinGlm::nbinfit(gsl_matrix *Y, gsl_matrix *X, gsl_matrix *O, gsl_matrix *B)
 }
 
 
-double PoissonGlm::getDisper( unsigned int id )const
+double PoissonGlm::getDisper( unsigned int id, double th )const
 {
     unsigned int i, df, nNonZero=0;
     double ss2, yij, mij, chi2=0;
@@ -660,10 +620,10 @@ double PoissonGlm::getDisper( unsigned int id )const
         yij = gsl_vector_get (&yj.vector, i);
         mij = gsl_vector_get (&mj.vector, i);
 	ss2 = (yij-mij)*(yij-mij); // ss = (y-mu)^2
-	if ( mij < eps ) mij = 1;
+	if ( mij < mintol ) mij = 1;
 	else  nNonZero++;	   
-        if ( varfunc(mij, phi[id])>eps )
-            chi2 = chi2 + ss2/varfunc(mij, phi[id]); // dist dependant
+        if ( varfunc(mij, th)>eps )
+            chi2 = chi2 + ss2/varfunc(mij, th); // dist dependant
     }
     if (nNonZero > nParams) 
         df = nNonZero - nParams; 
@@ -673,52 +633,101 @@ double PoissonGlm::getDisper( unsigned int id )const
 }
 
 
-double NBinGlm::llfunc ( double yi, double mui, double a  )const 
+double NBinGlm::llfunc ( double yi, double mui, double th  )const 
 {
-    double l=0, k=0, p=0;
-//    double eps=mmRef->tol;
+    double l=0, p=0;
 
-    if (a < 0 ) 
-       GSL_ERROR("Error in llfunc, phi should be non-negative", GSL_ERANGE); 
-    else if (a<eps) { // equivalent to Poisson log likelihood
-       if (yi < 0)
-          GSL_ERROR("Error in llfunc, y should be non-negative", GSL_ERANGE);
-       else if (yi<eps) l = -mui;
-       else l = (yi*log(mui)-mui-gsl_sf_lngamma(yi+1));
+    if (th==0) {
+       l = gsl_sf_lngamma(eps)+log(MAX(yi,eps)); 
+    }
+    else if (th>maxth) { // Poisson
+       l = -yi*log(mui) + mui + gsl_sf_lngamma(yi+1);
     }
     else {
-       if (yi < 0)
-          GSL_ERROR("Error in llfunc, y should be non-negative", GSL_ERANGE);
-       else if (yi<eps)  l = -(log(1+mui*a))/a;    
-       else {
-          k = 1/a;
-          l = gsl_sf_lngamma(yi+k)-gsl_sf_lngamma(k)-gsl_sf_lngamma(yi+1);
-          p = 1/(1+GSL_MAX(mui, eps)*a);
-          l = l + log(p)*k + yi*log( ((1-p)<eps) ? 1:(1-p) );  
-       }
+       l = (yi+th)*log(mui+th) - yi*log(mui) + gsl_sf_lngamma(yi+1) - th*log(th) + gsl_sf_lngamma(th) - gsl_sf_lngamma(yi+th); 
     }
 
-    return 2*l;
+    if (l!=l)  printf("l=nan, theta=%.4f, yi=%.4f, mu=%.4f\n", th, yi, mui);
+    
+    return -2*l;
+}
+
+
+double NBinGlm::thetaML(double t0, unsigned int id, unsigned int limit)
+{
+    // equivalent to theta.ml() in MASS
+    // Note that theta here is the dispersion parameter
+    // So phi = 1/theta;
+    unsigned int i, it=0;
+    double del=1, sum=1, num=0, th;
+    double score, info, yij, mij, dl, ddl;
+    if (t0==0) {
+       for (i=0; i<nRows; i++) {
+           yij = gsl_matrix_get(Yref, i, id);
+	   mij = gsl_matrix_get(Mu, i, id);
+	   if (mij>0) {
+	      sum = sum+(yij/mij-1)*(yij/mij-1);
+	      num = num+1;
+	   }
+       }
+       t0 = num/sum;
+    }
+    th = t0;
+    while ( (it<limit)&(ABS(del)>eps) ) {
+        it++;
+        score=0;
+	info=0;
+        for ( i=0; i<nRows; i++ ) {
+           yij = gsl_matrix_get(Yref, i, id);
+	   mij = gsl_matrix_get(Mu, i, id);
+           if (th==0) {
+//	      printf("Warning: th==0\n");
+              dl  = gsl_sf_psi(MAX(eps,yij))+1-log(mij)-yij/mij-gsl_sf_psi(eps)+log(eps); 
+	      ddl = -gsl_sf_psi_1(MAX(eps,yij))+2/(mij)-yij/mij*mij+gsl_sf_psi_1(eps)-1/eps; 
+	      
+	   }
+	   else {
+              dl  = gsl_sf_psi(yij+th)-gsl_sf_psi(th)+log(th)+1-log(mij+th)-(yij+th)/(mij+th); 
+	      ddl = -gsl_sf_psi_1(yij+th)+gsl_sf_psi_1(th)-1/th+2/(mij+th)-(yij+th)/((mij+th)*(mij+th)); // dl^2/d^2a
+	   }
+	   score = score + dl;
+	   info = info + ddl;
+	}   
+//	del = score/(GSL_SIGN(info)*MAX(ABS(info), mintol));
+        if (ABS(info) < mintol) info = GSL_SIGN(info)*mintol;
+	del = score/info;
+        th = th + del; // Normal Newton use - instead of + for -ddl
+        if (th<0) th = 0;
+        if (th!=th) printf("yij=%.2f, mij=%.4f, t0=%.4f, score=%.4f, info=%.4f, del=%.4f, theta=%.4f", yij, mij, t0, score, info, del, th);
+  	if (th>maxth) break;
+	if (th<0.01) break;
+    }
+
+    return th;
+    
 }
 
 
 int NBinGlm::getfAfAdash(double a, unsigned int id, double *fAPtr, double *fAdashPtr )
 {
+   // fA and fAdsh are equivalent to score and info in theta.ml(), MASS
     unsigned int i;
-    double yij, mij, dl, ddl, k=1/a;
+    double yij, mij, dl, ddl, th=1/a;
     *fAPtr = 0;
     *fAdashPtr = 0;
     for ( i=0; i<nRows; i++ ) {
         yij = gsl_matrix_get(Yref, i, id);
 	mij = gsl_matrix_get(Mu, i, id);
-        dl=gsl_sf_psi(yij+k)-gsl_sf_psi(k)-log(mij+k)+log(k)-(yij-mij)/(mij+k); // dl/da
+    //    dl=gsl_sf_psi(yij+k)-gsl_sf_psi(k)-log(mij+k)+log(k)-(yij-mij)/(mij+k); // dl/da
+        dl=gsl_sf_psi(yij+th)-gsl_sf_psi(th)+log(th)+1-log(mij+th)-(yij+th)/(mij+th); // dl/da
 	*fAPtr = *fAPtr + dl;  // sum
-	ddl=gsl_sf_psi_1(yij+k)-gsl_sf_psi_1(k)+a*mij/(mij+k)+(yij-mij)/(mij+k)/(mij+k); // dl^2/d^2a
-	*fAdashPtr = *fAdashPtr + ddl + 2*a*dl;	//sum
+//	ddl=gsl_sf_psi_1(yij+k)-gsl_sf_psi_1(k)+a*mij/(mij+k)+(yij-mij)/(mij+k)/(mij+k); // dl^2/d^2a
+	ddl=gsl_sf_psi_1(yij+th)-gsl_sf_psi_1(th)+a-2/(mij+th)+(yij+th)/((mij+th)*(mij+th)); // dl^2/d^2a
+	*fAdashPtr = *fAdashPtr + ddl;	//sum
     }
 
-    *fAPtr = - k*k*(*fAPtr);
-    *fAdashPtr = exp(4*log(k))*(*fAdashPtr);
+//    *fAPtr = - k*k*(*fAPtr);
+//    *fAdashPtr = exp(4*log(k))*(*fAdashPtr);
 
     return SUCCESS;
     
@@ -763,9 +772,9 @@ void glm::display(void)
 //    for ( j=0; j<nVars; j++ ) printf("%.2f ", aic[j]);	
 //    printf("\n");
 //    printf("# of convergence\n");    
-//    for ( j=0; j<nVars; j++ )
-//        printf("%d ", iterconv[j]); 
-//    printf("\n");	       
+    for ( j=0; j<nVars; j++ )
+        printf("%d ", iterconv[j]); 
+    printf("\n");	       
 //   printf("Residual deviance=\n " );
 //    for ( j=0; j<nVars; j++ ) printf("%.2f ", dev[j]);
 //    printf("\n");	       
